/*
    ACE expansion plugins definitions

    Copyright  2015-2016 Philippe Rimauro. All rights reserved.
*/

#ifndef LIBRARIES_ACEPANSION_H
#define LIBRARIES_ACEPANSION_H

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif
#ifndef INTUITION_CLASSUSR_H
#include <intuition/classusr.h>
#endif

/* Generic part of the plugin data structure */
struct ACEpansionPlugin
{
    ULONG   ap_APIVersion;     // Version API implemented by the plugin (always v1.0 for now)
    ULONG   ap_Flags;          // Flags to handle ports exclusive usage between plugins and activate needed API
                               // (use combinaison of ACE_FLAGSF_xxx here)
    STRPTR  ap_Title;          // Title for the plugin that will be used in the window title and the "Help" menu
    STRPTR  ap_HelpFileName;   // Name of the guide to be used from the "Help" menu (NULL if no help)
    STRPTR  ap_ToggleMenuName; // Label that will be used for the activate toggle from the "Expansions" menu
    STRPTR  ap_PrefsMenuName;  // Label that will be used for the (optional) preferences/control window from the "Expansions" menu
    Object *ap_PrefsWindow;    // MUI object that will be used for preferences/control window contents (NULL if none)
                               // Please note that the semaphore provided at CreatePlugin() time shall be used
                               // to access custom part of this data structure from the object code itself
                               // (object and library accessors are not handled by the same task)
                               // Also, don't forget to dispose your object at DeletePlugin()
};

/*
 * FLAGSB are bit definitions, FLAGSF are field definitions
 * To be used to set ap_Flags field in LibCreatePlugin() function
 */
#define ACE_FLAGSB_EXCLUSIVE_MOUSE    18 // Plugin must be exclusive regarding mouse emulation     ** V2 API only **
#define ACE_FLAGSB_EXCLUSIVE_JOYSTICK 17 // Plugin must be exclusive regarding joystick port usage ** V2 API only **
#define ACE_FLAGSB_EXCLUSIVE_PRINTER  16 // Plugin must be exclusive regarding printer port usage

#define ACE_FLAGSB_ACTIVE_JOYSTICK     8 // Plugin is using joysticks (activate GetJoysticks() API) ** V2 API only **
#define ACE_FLAGSB_ACTIVE_EMULATE      7 // Plugin is using real-time scheduling (activate Emulate() API)
#define ACE_FLAGSB_ACTIVE_RESET        6 // Plugin is using bus reset (activate Reset() API)
#define ACE_FLAGSB_ACTIVE_WRITEIO      5 // Plugin is using I/O ports write (activate WriteIO() API)
#define ACE_FLAGSB_ACTIVE_READIO       4 // Plugin is using I/O port read (activate ReadIO() API)
#define ACE_FLAGSB_ACTIVE_AUDIO        3 // Plugin is using CPC audio output (activate GetAudio() API)
#define ACE_FLAGSB_ACTIVE_PRINTER      2 // Plugin is using printer port (activate Printer() API)
#define ACE_FLAGSB_ACTIVE_CURSOR       1 // Plugin is using cursor trigger (activate Cursor() API)
#define ACE_FLAGSB_ACTIVE_ACKINT       0 // Plugin is using interrupt ack trigger (activate AcknowledgeInterrupt() API)

#define ACE_FLAGSF_EXCLUSIVE_MOUSE    (1<<ACE_FLAGSB_EXCLUSIVE_MOUSE)    // ** V2 API only **
#define ACE_FLAGSF_EXCLUSIVE_JOYSTICK (1<<ACE_FLAGSB_EXCLUSIVE_JOYSTICK) // ** V2 API only **
#define ACE_FLAGSF_EXCLUSIVE_PRINTER  (1<<ACE_FLAGSB_EXCLUSIVE_PRINTER)

#define ACE_FLAGSF_ACTIVE_JOYSTICK    (1<<ACE_FLAGSB_ACTIVE_JOYSTICK) // ** V2 API only **
#define ACE_FLAGSF_ACTIVE_EMULATE     (1<<ACE_FLAGSB_ACTIVE_EMULATE)
#define ACE_FLAGSF_ACTIVE_RESET       (1<<ACE_FLAGSB_ACTIVE_RESET)
#define ACE_FLAGSF_ACTIVE_WRITEIO     (1<<ACE_FLAGSB_ACTIVE_WRITEIO)
#define ACE_FLAGSF_ACTIVE_READIO      (1<<ACE_FLAGSB_ACTIVE_READIO)
#define ACE_FLAGSF_ACTIVE_AUDIO       (1<<ACE_FLAGSB_ACTIVE_AUDIO)
#define ACE_FLAGSF_ACTIVE_PRINTER     (1<<ACE_FLAGSB_ACTIVE_PRINTER)
#define ACE_FLAGSF_ACTIVE_CURSOR      (1<<ACE_FLAGSB_ACTIVE_CURSOR)
#define ACE_FLAGSF_ACTIVE_ACKINT      (1<<ACE_FLAGSB_ACTIVE_ACKINT)

/*
 * SIGNALB are bit definitions, SIGNALF are field definitions
 * To be used to alter signals field from plugin LibEmulate() API
 */
#define ACE_SIGNALB_INT 0 // Expansion port /INT signal

#define ACE_SIGNALF_INT (1<<ACE_SIGNALB_INT)

/*
 * ** V2 API only **
 * IODATAB are bit definitions, IODATAF are field definitions
 * To be used to set ioData in the Joystick() function
 * Warning: bits should be set to 0 (not 1!) to activate the
 *          direction and left unchanged otherwise.
 */
#define ACE_IODATAB_JOYSTICK_UP     0
#define ACE_IODATAB_JOYSTICK_DOWN   1
#define ACE_IODATAB_JOYSTICK_LEFT   2
#define ACE_IODATAB_JOYSTICK_RIGHT  3
#define ACE_IODATAB_JOYSTICK_FIRE1  4
#define ACE_IODATAB_JOYSTICK_FIRE2  5
#define ACE_IODATAB_JOYSTICK_FIRE3  6
#define ACE_IODATAB_JOYSTICK_PAUSE  7

#define ACE_IODATAF_JOYSTICK_UP    (1<<ACE_IODATAB_JOYSTICK_UP)
#define ACE_IODATAF_JOYSTICK_DOWN  (1<<ACE_IODATAB_JOYSTICK_DOWN)
#define ACE_IODATAF_JOYSTICK_LEFT  (1<<ACE_IODATAB_JOYSTICK_LEFT)
#define ACE_IODATAF_JOYSTICK_RIGHT (1<<ACE_IODATAB_JOYSTICK_RIGHT)
#define ACE_IODATAF_JOYSTICK_FIRE1 (1<<ACE_IODATAB_JOYSTICK_FIRE1)
#define ACE_IODATAF_JOYSTICK_FIRE2 (1<<ACE_IODATAB_JOYSTICK_FIRE2)
#define ACE_IODATAF_JOYSTICK_FIRE3 (1<<ACE_IODATAB_JOYSTICK_FIRE3)
#define ACE_IODATAF_JOYSTICK_PAUSE (1<<ACE_IODATAB_JOYSTICK_PAUSE)

#endif /* LIBRARIES_ACEPANSION_H */
